@@ -188,7 +188,8 @@ sub _fetch {
   if (!$self->{value_set}) {
     if (!$self->{job}->is_complete) {
       my $pid = Forks::Super::waitpid $self->{job_id}, WREAP_BG_OK;
-      if ($pid != $self->{job}->{real_pid}) {
+      if ($pid != $self->{job}->{real_pid} && $pid != $self->{job}->{pid}) {
+
 	carp "Forks::Super::bg_eval: ",
 	  "failed to retrieve result from process!\n";
 	$self->{value_set} = 1;
